import { CodeAction, CodeActionParams, Diagnostic, IConnection } from "vscode-languageserver";
import { URI } from "vscode-uri";
import { Settings } from "../../util/settings";
export declare const CODE_ACTION_ELM_MAKE: string;
export interface IElmCompilerError {
    type: string;
    errors: IError[];
}
export interface IElmError {
    title: string;
    type: string;
    path: string;
    message: Array<string | IStyledString>;
}
export interface IError {
    path: string | null;
    name: string;
    problems: IProblem[];
}
export interface IProblem {
    title: string;
    region: {
        start: {
            line: number;
            column: number;
        };
        end: {
            line: number;
            column: number;
        };
    };
    message: Array<string | IStyledString>;
}
export interface IStyledString {
    bold: boolean;
    underline: boolean;
    color: string;
    string: string;
}
export declare class ElmMakeDiagnostics {
    private connection;
    private elmWorkspaceFolder;
    private settings;
    constructor(connection: IConnection, elmWorkspaceFolder: URI, settings: Settings);
    createDiagnostics: (filePath: URI) => Promise<Map<string, Diagnostic[]>>;
    onCodeAction(params: CodeActionParams): CodeAction[];
    private convertDiagnosticsToCodeActions;
    private createQuickFix;
    private filterElmMakeDiagnostics;
    private checkForErrors;
}
