"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const globby_1 = __importDefault(require("globby"));
const vscode_uri_1 = require("vscode-uri");
const capabilityCalculator_1 = require("./capabilityCalculator");
const elmWorkspace_1 = require("./elmWorkspace");
const settings_1 = require("./util/settings");
class Server {
    constructor(connection, params, parser) {
        this.connection = connection;
        this.params = params;
        this.parser = parser;
        this.elmWorkspaceMap = new Map();
        this.calculator = new capabilityCalculator_1.CapabilityCalculator(params.capabilities);
        const initializationOptions = this.params.initializationOptions || {};
        this.settings = new settings_1.Settings(this.connection, initializationOptions);
        const uri = this.getWorkspaceUri(params);
        if (uri) {
            // Cleanup the path on windows, as globby does not like backslashes
            const globUri = uri.fsPath.replace(/\\/g, "/");
            const elmJsonGlob = `${globUri}/**/elm.json`;
            const elmJsons = globby_1.default.sync([
                elmJsonGlob,
                "!**/node_modules/**",
                "!**/elm-stuff/**",
            ]);
            if (elmJsons.length > 0) {
                connection.console.info(`Found ${elmJsons.length} elm.json files for workspace ${globUri}`);
                const listOfElmJsonFolders = elmJsons.map(a => this.getElmJsonFolder(a));
                const topLevelElmJsons = this.findTopLevelFolders(listOfElmJsonFolders);
                connection.console.info(`Found ${topLevelElmJsons.size} unique elmWorkspaces for workspace ${globUri}`);
                topLevelElmJsons.forEach(elmWorkspace => {
                    this.elmWorkspaceMap.set(elmWorkspace.toString(), new elmWorkspace_1.ElmWorkspace(elmWorkspace, connection, this.settings, this.parser));
                });
            }
            else {
                this.connection.console.info(`No elm.json found`);
            }
        }
        else {
            this.connection.console.info(`No workspace was setup by the client`);
        }
    }
    get capabilities() {
        return {
            capabilities: this.calculator.capabilities,
        };
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.elmWorkspaceMap.forEach(it => it.init());
        });
    }
    registerInitializedProviders() {
        return __awaiter(this, void 0, void 0, function* () {
            // We can now query the client for up to date settings
            this.settings.initFinished();
            this.elmWorkspaceMap.forEach(it => it.registerInitializedProviders());
        });
    }
    getElmJsonFolder(uri) {
        return vscode_uri_1.URI.file(uri.replace("elm.json", ""));
    }
    findTopLevelFolders(listOfElmJsonFolders) {
        const result = new Map();
        listOfElmJsonFolders.forEach(element => {
            result.set(element.toString(), element);
        });
        listOfElmJsonFolders.forEach(a => {
            listOfElmJsonFolders.forEach(b => {
                if (b.toString() !== a.toString() &&
                    b.toString().startsWith(a.toString())) {
                    result.delete(b.toString());
                }
            });
        });
        return result;
    }
    getWorkspaceUri(params) {
        if (params.rootUri) {
            return vscode_uri_1.URI.parse(params.rootUri);
        }
        else if (params.rootPath) {
            return vscode_uri_1.URI.file(params.rootPath);
        }
        else {
            return null;
        }
    }
}
exports.Server = Server;
//# sourceMappingURL=server.js.map